@@ -400,6 +400,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 #endif
 		    (node->type == XML_HTML_DOCUMENT_NODE))
 		    continue;
+		if ((node->type == XML_ELEMENT_NODE) && (node->name[0] == ' '))
+		    continue;
 		return(0);
             case XSLT_OP_ELEM:
 		if (node->type != XML_ELEMENT_NODE)
@@ -609,6 +611,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			xmlChar *query;
 			xmlXPathObjectPtr newlist;
 			xmlNodePtr parent = node->parent;
+			xmlDocPtr olddoc;
+			xmlNodePtr oldnode;
 
 			if (comp->pattern[0] == '/')
 			    query = xmlStrdup(comp->pattern);
@@ -616,7 +620,13 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			    query = xmlStrdup((const xmlChar *)"//");
 			    query = xmlStrcat(query, comp->pattern);
 			}
+			oldnode = ctxt->xpathCtxt->node;
+			olddoc = ctxt->xpathCtxt->doc;
+			ctxt->xpathCtxt->node = node;
+			ctxt->xpathCtxt->doc = doc;
 			newlist = xmlXPathEval(query, ctxt->xpathCtxt);
+			ctxt->xpathCtxt->node = oldnode;
+			ctxt->xpathCtxt->doc = olddoc;
 			xmlFree(query);
 			if (newlist == NULL)
 			    return(-1);
@@ -628,11 +638,10 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			if ((parent == NULL) || (node->doc == NULL))
 			    nocache = 1;
 			else {
-			    while (parent->parent != NULL)
-				parent = parent->parent;
-			    if (((parent->type != XML_DOCUMENT_NODE) &&
-				 (parent->type != XML_HTML_DOCUMENT_NODE)) ||
-				 (parent != (xmlNodePtr) node->doc))
+			    if ((doc->name != NULL) &&
+				(doc->name[0] == ' ') &&
+				(xmlStrEqual(BAD_CAST doc->name, 
+					     BAD_CAST " fake node libxslt")))
 				nocache = 1;
 			}
 			if (nocache == 0) {
@@ -702,6 +711,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			    if (sibling == previous)
 				break;
 			    if ((node->type == XML_ELEMENT_NODE) &&
+				(node->name != NULL) &&
+				(sibling->name != NULL) &&
 				(node->name[0] == sibling->name[0]) &&
 				(xmlStrEqual(node->name, sibling->name))) {
 				if ((select->value2 == NULL) ||
@@ -756,8 +767,10 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 				if (siblings == node) {
 				    len++;
 				    pos = len;
-				} else if ((node->name[0] == siblings->name[0])
-			       && (xmlStrEqual(node->name, siblings->name))) {
+				} else if ((node->name != NULL) &&
+					   (siblings->name != NULL) &&
+				    (node->name[0] == siblings->name[0]) &&
+				    (xmlStrEqual(node->name, siblings->name))) {
 				    if ((select->value2 == NULL) ||
 					((siblings->ns != NULL) &&
 					 (xmlStrEqual(select->value2,
@@ -1338,7 +1351,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 	if (CUR == '*') {
 	    NEXT;
 	    PUSH(XSLT_OP_ATTR, NULL, NULL);
-	    return;
+	    goto parse_predicate;
 	}
 	token = xsltScanQName(ctxt, &prefix);
 	if (prefix != NULL) {
@@ -1523,7 +1536,7 @@ parse_predicate:
 	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileStepPattern : ']' expected\n");
 	    ctxt->error = 1;
-	    goto error;
+	    return;
         }
 	ret = xmlStrndup(q, CUR_PTR - q);
 	PUSH(XSLT_OP_PREDICATE, ret, NULL);
@@ -1750,7 +1763,7 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 	    xsltTransformError(NULL, style, node,
 			     "xsltCompilePattern : failed to compile '%s'\n",
 			     element->pattern);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	    goto error;
 	}
 
@@ -1811,7 +1824,7 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
     if (end == 0) {
 	xsltTransformError(NULL, style, node,
 			 "xsltCompilePattern : NULL pattern\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	goto error;
     }
 
@@ -2037,6 +2050,8 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	     */
 	    switch (node->type) {
 		case XML_ELEMENT_NODE:
+		    if (node->name[0] == ' ')
+			break;
 		case XML_ATTRIBUTE_NODE:
 		case XML_PI_NODE:
 		    name = node->name;
@@ -2088,7 +2103,10 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	 */
 	switch (node->type) {
 	    case XML_ELEMENT_NODE:
-		list = curstyle->elemMatch;
+		if (node->name[0] == ' ')
+		    list = curstyle->rootMatch;
+		else
+		    list = curstyle->elemMatch;
 		break;
 	    case XML_ATTRIBUTE_NODE:
 		list = curstyle->attrMatch;
@@ -2151,6 +2169,19 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 		}
 		list = list->next;
 	    }
+	} else if ((node->type == XML_PI_NODE) ||
+		   (node->type == XML_COMMENT_NODE)) {
+	    list = curstyle->elemMatch;
+	    while ((list != NULL) &&
+		   ((ret == NULL)  || (list->priority > priority))) {
+		if (xsltTestCompMatch(ctxt, list, node,
+				      ctxt->mode, ctxt->modeURI)) {
+		    ret = list->template;
+		    priority = list->priority;
+		    break;
+		}
+		list = list->next;
+	    }
 	}
 
 	if (node->_private != NULL) {
@@ -2217,36 +2248,3 @@ xsltFreeTemplateHashes(xsltStylesheetPtr style) {
         xsltFreeCompMatchList(style->commentMatch);
 }
 
-#if 0
-/**
- * xsltMatchPattern
- * @node: a node in the source tree
- * @pattern: an XSLT pattern
- * @ctxtdoc:  context document (for namespaces)
- * @ctxtnode:  context node (for namespaces)
- *
- * Determine if a node matches a pattern.
- */
-int
-xsltMatchPattern(xsltTransformContextPtr context,
-		 xmlNodePtr node,
-		 const xmlChar *pattern,
-		 xmlDocPtr ctxtdoc,
-		 xmlNodePtr ctxtnode)
-{
-    int match = 0;
-    xsltCompMatchPtr first, comp;
-
-    if ((context != NULL) && (pattern != NULL)) {
-	first = xsltCompilePattern(pattern, ctxtdoc, ctxtnode);
-	for (comp = first; comp != NULL; comp = comp->next) {
-	    match = xsltTestCompMatch(context, comp, node, NULL, NULL);
-	    if (match)
-		break; /* for */
-	}
-	if (first)
-	    xsltFreeCompMatchList(first);
-    }
-    return match;
-}
-#endif
