@@ -894,11 +894,24 @@ xsltDefaultSortFunction(xsltTransformContextPtr ctxt, xmlNodePtr *sorts,
 			res = resultsTab[depth];
 			if (res == NULL) 
 			    break;
-			if (res[j] == NULL)
-			    tst = 1;
-			else {
+			if (res[j] == NULL) {
+			    if (res[j+incr] != NULL)
+				tst = 1;
+			} else {
 			    if (numb) {
-				if (res[j]->floatval == res[j + incr]->floatval)
+				/* We make NaN smaller than number in
+				   accordance with XSLT spec */
+				if (xmlXPathIsNaN(res[j]->floatval)) {
+				    if (xmlXPathIsNaN(res[j +
+				    		incr]->floatval))
+					tst = 0;
+				    else
+				        tst = -1;
+				} else if (xmlXPathIsNaN(res[j + incr]->
+						floatval))
+				    tst = 1;
+				else if (res[j]->floatval == res[j + incr]->
+						floatval)
 				    tst = 0;
 				else if (res[j]->floatval > 
 					res[j + incr]->floatval)
@@ -1048,7 +1061,6 @@ int
 xsltSaveResultTo(xmlOutputBufferPtr buf, xmlDocPtr result,
 	       xsltStylesheetPtr style) {
     const xmlChar *encoding;
-    xmlNodePtr root;
     int base;
     const xmlChar *method;
     int indent;
@@ -1077,11 +1089,6 @@ xsltSaveResultTo(xmlOutputBufferPtr buf, xmlDocPtr result,
     if ((method == NULL) && (result->type == XML_HTML_DOCUMENT_NODE))
 	method = (const xmlChar *) "html";
 
-    if (method == NULL)
-	root = xmlDocGetRootElement(result);
-    else
-	root = NULL;
-
     if ((method != NULL) &&
 	(xmlStrEqual(method, (const xmlChar *) "html"))) {
 	if (encoding != NULL) {
@@ -1146,11 +1153,9 @@ xsltSaveResultTo(xmlOutputBufferPtr buf, xmlDocPtr result,
     } else {
 	int omitXmlDecl;
 	int standalone;
-	const xmlChar *version;
 
 	XSLT_GET_IMPORT_INT(omitXmlDecl, style, omitXmlDeclaration);
 	XSLT_GET_IMPORT_INT(standalone, style, standalone);
-	XSLT_GET_IMPORT_PTR(version, style, version)
 
 	if (omitXmlDecl != 1) {
 	    xmlOutputBufferWriteString(buf, "<?xml version=");
@@ -1620,7 +1625,7 @@ xsltGetProfileInformation(xsltTransformContextPtr ctxt)
     xsltStylesheetPtr style;
     xsltTemplatePtr *templates;
     xsltTemplatePtr templ;
-    int nb = 0, max = 0, i, j, total, totalt;
+    int nb = 0, max = 0, i, j;
 
     if (!ctxt)
         return NULL;
@@ -1675,8 +1680,6 @@ xsltGetProfileInformation(xsltTransformContextPtr ctxt)
     root = xmlNewDocNode(ret, NULL, BAD_CAST "profile", NULL);
     xmlDocSetRootElement(ret, root);
 
-    total = 0;
-    totalt = 0;
     for (i = 0; i < nb; i++) {
         child = xmlNewChild(root, NULL, BAD_CAST "template", NULL);
         sprintf(buf, "%d", i + 1);
